import dash
from dash import Dash, dcc, html

from amora.dash.authentication import add_auth0_login
from amora.dash.components import side_bar
from amora.dash.config import settings
from amora.dash.css_styles import styles
from amora.models import list_models

dash_app = Dash(
    __name__, external_stylesheets=settings.external_stylesheets, use_pages=True
)

if settings.auth0_login_enabled:
    add_auth0_login(dash_app)

# App
dash_app.layout = html.Div(
    style=styles["container"],
    children=[
        html.Div(
            [
                dcc.Location(id="url"),
                side_bar.component(),
                html.Div(
                    dash.page_container,
                    style={
                        "margin-left": "24rem",
                        "margin-right": "2rem",
                        "padding": "2rem 1rem",
                        "overflow": "scroll",
                    },
                    id="page-content",
                ),
            ],
        ),
    ],
)

# fixme: should be in a post init callback or shouldn't be needed at all
list(list_models())



import dash_bootstrap_components as dbc
import plotly.express as px
from dash import dash_table, dcc, html
from dash.development.base_component import Component

from amora.questions import Question
from amora.visualization import BarChart, BigNumber, LineChart, PieChart, Visualization


def answer_visualization(visualization: Visualization) -> Component:
    view_config = visualization.config

    df = visualization.data
    if df.empty:
        return dbc.Alert("⚠️ No data", color="warning")

    if isinstance(view_config, BigNumber):
        big_number = view_config.value_func(df)
        return html.H4(big_number)
    elif isinstance(view_config, LineChart):
        return dcc.Graph(
            figure=px.line(
                df,
                x=view_config.x_func(df),
                y=view_config.y_func(df),
            )
        )
    elif isinstance(view_config, BarChart):
        return dcc.Graph(
            figure=px.bar(
                df,
                x=view_config.x_func(df),
                y=view_config.y_func(df),
            )
        )
    elif isinstance(view_config, PieChart):
        return dcc.Graph(
            figure=px.pie(df, values=view_config.values, names=view_config.names)
        )
    else:
        return dash_table.DataTable(
            columns=[
                {"name": col, "id": col, "selectable": True}
                for col in df.columns.values
            ],
            data=df.to_dict("records"),
            row_selectable="multi",
            sort_action="native",
            style_cell={
                "overflow": "hidden",
                "textOverflow": "ellipsis",
                "maxWidth": 0,
            },
            export_format="csv",
            export_headers="display",
        )


def component(question: Question) -> Component:
    return dbc.Card(
        className="question-card",
        children=dbc.CardBody(
            [
                html.H5(question.name, className="card-title"),
                answer_visualization(question.render()),
                dbc.Accordion(
                    [
                        dbc.AccordionItem(
                            html.Code(question.sql, lang="sql"), title="SQL"
                        )
                    ],
                    start_collapsed=True,
                ),
            ]
        ),
    )
